require 'spec_helper'

describe Social::Engagement do
  subject { Social::Engagement }
  let(:url) { 'www.example.com' }

  describe '::for' do
    let(:url) { 'www.studentbeans.com/offers/offers/1' }
    let(:offer) { Offer.new }
    let(:social_engagement) { SocialEngagement.new(url: url) }

    it 'creates an url for the object and gets the engagement' do
      Social::Url.should_receive(:for).with(offer).and_return(url)
      Social::Engagement.should_receive(:for_url).with(url).and_return(social_engagement)

      Social::Engagement.for(offer).should == social_engagement
    end
  end

  describe '::for_url' do
    let(:cached_social_engagement) { SocialEngagement.new(url: url, twitter: 1, facebook: 1) }
    let(:social_engagement_from_web) { SocialEngagement.new(url: url, twitter: 2, facebook: 2) }

    context 'this social engagement is cached' do
      context 'and is valid' do
        it 'returns it from the cache' do
          SocialEngagement.should_receive(:for_url).with(url).and_return(cached_social_engagement)
          Social::InvalidationPolicy.should_receive(:valid?).with(cached_social_engagement).and_return(true)

          subject.for_url(url).should == cached_social_engagement
        end
      end

      context 'and is not valid' do
        it 'gets it from the web and updates the cache' do
          SocialEngagement.should_receive(:for_url).with(url).and_return(cached_social_engagement)
          Social::InvalidationPolicy.should_receive(:valid?).with(cached_social_engagement).and_return(false)
          subject.should_receive(:from_web).and_return(social_engagement_from_web)
          cached_social_engagement.should_receive(:update_counts).with(social_engagement_from_web)

          subject.for_url(url).should == cached_social_engagement
        end
      end
    end

    context 'this social engagement is not cached' do
      it 'gets it from the web and caches it' do
        SocialEngagement.should_receive(:for_url).with(url).and_raise(ActiveRecord::RecordNotFound)
        subject.should_receive(:from_web).and_return(social_engagement_from_web)
        social_engagement_from_web.should_receive(:save)

        subject.for_url(url).should == social_engagement_from_web
      end
    end
  end

  describe '::from_web' do
    let(:facebook_engagement) { 100 }
    let(:twitter_engagement) { 20 }

    it 'builds a SocialEngagement for a given url' do
      Social::Facebook.should_receive(:total_count).with(url).and_return(facebook_engagement)
      Social::Twitter.should_receive(:total_count).with(url).and_return(twitter_engagement)

      social_engagement = subject.from_web(url)
      social_engagement.facebook.should == facebook_engagement
      social_engagement.twitter.should == twitter_engagement
    end
  end
end
