require 'spec_helper'

describe SocialEngagement do

  it { should validate_presence_of(:url) }

  describe '::for_url' do
    let(:url) { 'www.example.com' }

    let!(:social_engagement) { SocialEngagement.create(url: url) }

    it 'finds a social engagement by url' do
      SocialEngagement.for_url(url).should == social_engagement
    end

    it 'raises exception if the social engagement is not found' do
      expect do
        SocialEngagement.for_url('unknown url')
      end.to raise_error(ActiveRecord::RecordNotFound)
    end
  end

  describe '#update_counts' do
    it 'updates an instance with the engagement count attributes of another instance' do
      social_engagement = SocialEngagement.new(url: 'www.example.com', facebook: 0, twitter: 0)
      social_engagement.update_counts(SocialEngagement.new(facebook: 20, twitter: 10))
      social_engagement.facebook.should == 20
      social_engagement.twitter.should == 10
    end
  end
end
